import logging
import re
import xmltodict

from constants import (
    TAB_SEPARATOR,
    PENTAHO_USERS_ENDPOINT_API,
    CREATE_USERS,
    LIST_USERS,
    CHANGE_USER_PASSWORD,
    DELETE_USERS,
    USERS_USERNAME_VAR,
    USERS_PASSWORD_VAR,
    USERS_NEW_PASSWORD_VAR,
    USERS_OLD_PASSWORD_VAR,
    USERS_DELETE_VAR
)

from penapi.base_api import PentahoBaseAPI

try:
    logger = logging.getLogger(__name__)
except Exception:
    logger = logging.getLogger("PENTAHO_USERS_API")


class PentahoUsersAPI(PentahoBaseAPI):

    def list(self, regex=None):
        """
        Get list of all users
        :param regex: filter pattern
        :return: success/failure
        :rtype: list
        """
        user_list = list()
        response = self._pentaho.make_call(PENTAHO_USERS_ENDPOINT_API, LIST_USERS)
        if response.status_code == 200:
            try:
                user_list = xmltodict.parse(response.text)['userList']['users']
            except Exception, e:
                logger.exception(e)
        if regex:
            user_list = [v for v in user_list if re.match(regex, v)]
        return user_list

    def create(self, username=None, password=None):
        """
        Create a user in pentaho
        :param username: the username to assign to the user
        :param password: the password for the user
        :return: success/failure
        :rtype: boolean
        """
        if not username or not password:
            raise ValueError("[ERROR] user creation parameters missing... ")
        user_credentials = {
            USERS_USERNAME_VAR: username,
            USERS_PASSWORD_VAR: password,
        }
        response = self._pentaho.make_call(PENTAHO_USERS_ENDPOINT_API, CREATE_USERS,
                                           json=user_credentials)
        return response.status_code == 200

    def change_password(self, username, old_password, new_password):
        """
        Change password for a user in pentaho
        :param username: user to change the password
        :param old_password: old password
        :param new_password: new password
        :return: success/failure
        :rtype: boolean
        """
        user_password_change = {
            USERS_USERNAME_VAR: username,
            USERS_OLD_PASSWORD_VAR: old_password,
            USERS_NEW_PASSWORD_VAR: new_password
        }
        response = self._pentaho.make_call(PENTAHO_USERS_ENDPOINT_API, CHANGE_USER_PASSWORD,
                                           json=user_password_change)
        return response.status_code == 200

    def delete(self, usernames=None):
        """
        Delete a user in pentaho
        :param usernames: the usernames to delete (string or list)
        :return: success/failure
        :rtype: boolean
        """
        if not usernames:
            raise ValueError("[ERROR] user delete parameters missing... ")
        if type(usernames) is list:
            usernames = TAB_SEPARATOR.join(usernames)
        response = self._pentaho.make_call(PENTAHO_USERS_ENDPOINT_API, DELETE_USERS,
                                           params={USERS_DELETE_VAR: usernames})
        return response.status_code == 200



import pytest
import requests_mock
import os

from penapi.pentaho import Pentaho
from tests import FIXTURES_DIR


def list_roles():
    with open(os.path.join(FIXTURES_DIR, 'roles_api.xml'), 'r') as fixture_file:
        data = fixture_file.read().replace('\n', '')
    return data


def list_one_role():
    with open(os.path.join(FIXTURES_DIR, 'one_role_api.xml'), 'r') as fixture_file:
        data = fixture_file.read().replace('\n', '')
    return data


def list_users():
    with open(os.path.join(FIXTURES_DIR, 'users_api.xml'), 'r') as fixture_file:
        data = fixture_file.read().replace('\n', '')
    return data


def list_perm_role_map():
    with open(os.path.join(FIXTURES_DIR, 'perm_role_map.xml'), 'r') as fixture_file:
        data = fixture_file.read().replace('\n', '')
    return data


@pytest.fixture
def pentaho():
    return Pentaho(pentaho_base_url='http://test.com')


@requests_mock.Mocker(kw='mock_api')
def test_role_api_list_user_role_success(mock_api, pentaho=pentaho()):
    mock_api.get('http://test.com/pentaho/api/userroledao/userRoles?userName=test', text=list_roles())
    roles_list = pentaho.roles.list_for_user('test')
    assert len(roles_list) == 3


@requests_mock.Mocker(kw='mock_api')
def test_role_api_list_user_one_success(mock_api, pentaho=pentaho()):
    mock_api.get('http://test.com/pentaho/api/userroledao/userRoles?userName=test', text=list_one_role())
    roles_list = pentaho.roles.list_for_user('test')
    assert len(roles_list) == 1


@requests_mock.Mocker(kw='mock_api')
def test_role_api_list_user_fail(mock_api, pentaho=pentaho()):
    mock_api.get('http://test.com/pentaho/api/userroledao/userRoles?userName=test', text='', status_code=500)
    roles_list = pentaho.roles.list_for_user('test')
    assert len(roles_list) == 0


@requests_mock.Mocker(kw='mock_api')
def test_role_api_assign_success(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/assignRoleToUser?userName=test&roleNames=power', text='')
    success = pentaho.roles.assign_to_user('test', ['power'])
    assert success


@requests_mock.Mocker(kw='mock_api')
def test_role_api_assign_fail(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/assignRoleToUser?userName=test&roleNames=power',
                 text='', status_code=403)
    success = pentaho.roles.assign_to_user('test', ['power'])
    assert not success


def test_role_api_assign_fail_2(pentaho=pentaho()):
    with pytest.raises(ValueError):
        pentaho.roles.assign_to_user('test', 'power')


@requests_mock.Mocker(kw='mock_api')
def test_role_api_remove_success(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/removeRoleFromUser?userName=test&roleNames=Business', text='')
    success = pentaho.roles.remove_from_user('test', ['Business'])
    assert success


@requests_mock.Mocker(kw='mock_api')
def test_role_api_remove_fail(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/removeRoleFromUser?userName=test&roleNames=Business',
                 text='', status_code=403)
    success = pentaho.roles.remove_from_user('test', ['Business'])
    assert not success


def test_role_api_remove_fail_2(pentaho=pentaho()):
    with pytest.raises(ValueError):
        pentaho.roles.remove_from_user('test', 'Business')


@requests_mock.Mocker(kw='mock_api')
def test_role_api_create_role_success(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/createRole?roleName=test', text='')
    success = pentaho.roles.create('test')
    assert success


@requests_mock.Mocker(kw='mock_api')
def test_role_api_create_role_fail(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/createRole?roleName=test', text='', status_code=403)
    success = pentaho.roles.create('test')
    assert not success


@requests_mock.Mocker(kw='mock_api')
def test_role_api_delete_roles_success(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/deleteRoles?roleNames=test', text='')
    success = pentaho.roles.delete('test')
    assert success


@requests_mock.Mocker(kw='mock_api')
def test_role_api_delete_roles_list_success(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/deleteRoles?roleNames=test1%09test2', text='')
    success = pentaho.roles.delete(['test1', 'test2'])
    assert success


@requests_mock.Mocker(kw='mock_api')
def test_role_api_delete_roles_fail(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/deleteRoles?roleNames=test', text='', status_code=403)
    success = pentaho.roles.delete('test')
    assert not success


@requests_mock.Mocker(kw='mock_api')
def test_role_api_list_success(mock_api, pentaho=pentaho()):
    mock_api.get('http://test.com/pentaho/api/userroledao/roles', text=list_roles())
    roles_list = pentaho.roles.list()
    assert len(roles_list) == 3


@requests_mock.Mocker(kw='mock_api')
def test_role_api_list_fail(mock_api, pentaho=pentaho()):
    mock_api.get('http://test.com/pentaho/api/userroledao/roles', text='', status_code=500)
    roles_list = pentaho.roles.list()
    assert len(roles_list) == 0


@requests_mock.Mocker(kw='mock_api')
def test_role_api_list_user_success(mock_api, pentaho=pentaho()):
    mock_api.get('http://test.com/pentaho/api/userroledao/roleMembers?roleName=test', text=list_users())
    user_list = pentaho.roles.list_members('test')
    assert len(user_list) == 6


@requests_mock.Mocker(kw='mock_api')
def test_role_api_list_user_fail(mock_api, pentaho=pentaho()):
    mock_api.get('http://test.com/pentaho/api/userroledao/roleMembers?roleName=test', text='', status_code=500)
    user_list = pentaho.roles.list_members('test')
    assert len(user_list) == 0


@requests_mock.Mocker(kw='mock_api')
def test_role_api_assign_perm_success(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/roleAssignments', text='')
    success = pentaho.roles.assign_permissions_to_role('test1', read=True, create=True)
    assert success


@requests_mock.Mocker(kw='mock_api')
def test_role_api_list_perm_role_map_success(mock_api, pentaho=pentaho()):
    mock_api.get('http://test.com/pentaho/api/userroledao/logicalRoleMap?locale=en', text=list_perm_role_map())
    perm_role_map = pentaho.roles.get_permissions_role_map()
    assert len(perm_role_map['assignments']) == 2
    assert len(perm_role_map['localizedRoleNames']) == 7


@requests_mock.Mocker(kw='mock_api')
def test_role_api_list_perm_role_map_fail(mock_api, pentaho=pentaho()):
    mock_api.get('http://test.com/pentaho/api/userroledao/logicalRoleMap?locale=en', text='', status_code=403)
    perm_role_map = pentaho.roles.get_permissions_role_map()
    assert not perm_role_map



import os

import pytest
import requests_mock

from penapi.pentaho import Pentaho
from tests import FIXTURES_DIR


@pytest.fixture
def list_users():
    with open(os.path.join(FIXTURES_DIR, 'users_api.xml'), 'r') as fixture_file:
        data = fixture_file.read().replace('\n', '')
    return data


@pytest.fixture
def pentaho():
    pentaho = Pentaho(pentaho_base_url='http://test.com')
    return pentaho


@requests_mock.Mocker(kw='mock_api')
def test_user_api_list_success(mock_api, pentaho=pentaho()):
    mock_api.get('http://test.com/pentaho/api/userroledao/users', text=list_users())
    user_list = pentaho.users.list()
    assert len(user_list) == 6


@requests_mock.Mocker(kw='mock_api')
def test_user_api_list_fail(mock_api, pentaho=pentaho()):
    mock_api.get('http://test.com/pentaho/api/userroledao/users', text='', status_code=500)
    user_list = pentaho.users.list()
    assert len(user_list) == 0


@requests_mock.Mocker(kw='mock_api')
def test_user_api_create_success(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/createUser', text='')
    success = pentaho.users.create('test', 'pass')
    assert success


@requests_mock.Mocker(kw='mock_api')
def test_user_api_create_fail(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/createUser', text='', status_code=412)
    success = pentaho.users.create('test', 'pass')
    assert not success


def test_user_api_create_fail_2(pentaho=pentaho()):
    with pytest.raises(ValueError):
        pentaho.users.create(username='test')


@requests_mock.Mocker(kw='mock_api')
def test_user_api_change_password_success(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/user', text='')
    success = pentaho.users.change_password('test', 'pass', 'pass')
    assert success


@requests_mock.Mocker(kw='mock_api')
def test_user_api_change_password_fail(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/user', text='', status_code=403)
    success = pentaho.users.change_password('test', 'pass1', 'pass2')
    assert not success


@requests_mock.Mocker(kw='mock_api')
def test_user_api_delete_users_success(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/deleteUsers?userNames=test', text='')
    success = pentaho.users.delete('test')
    assert success


@requests_mock.Mocker(kw='mock_api')
def test_user_api_delete_users_list_success(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/deleteUsers?userNames=test1%09test2', text='')
    success = pentaho.users.delete(['test1', 'test2'])
    assert success


@requests_mock.Mocker(kw='mock_api')
def test_user_api_delete_users_fail(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/userroledao/deleteUsers?userNames=test', text='', status_code=403)
    success = pentaho.users.delete('test')
    assert not success



import logging
import re
import xmltodict

from constants import (
    TAB_SEPARATOR,
    PENTAHO_ROLES_ENDPOINT_API,
    USERS_USERNAME_VAR,
    ROLES_ASSIGN_VAR,
    ROLE_CREATE_VAR,
    ROLE_DELETE_VAR,
    ROLE_LIST_MEMBERS_VAR,
    ROLE_MAP_LOCALE_VAR,
    GET_ROLES_FOR_USER,
    ASSIGN_ROLE_TO_USER,
    REMOVE_ROLE_FROM_USER,
    CREATE_ROLE,
    DELETE_ROLE,
    LIST_ROLES,
    LIST_MEMBERS_ROLE,
    ASSIGN_PERM_TO_ROLE,
    LIST_PERM_ROLE_MAP
)

from penapi.base_api import PentahoBaseAPI

logger = logging.getLogger(__name__)


class PentahoRolesAPI(PentahoBaseAPI):

    def list_for_user(self, username):
        """
        Get roles for a user
        :param username: user to get roles
        :return: success/failure
        :rtype: list
        """
        role_list = list()
        response = self._pentaho.make_call(PENTAHO_ROLES_ENDPOINT_API, GET_ROLES_FOR_USER,
                                           params={USERS_USERNAME_VAR: username})
        if response.status_code == 200:
            try:
                if isinstance(xmltodict.parse(response.text)['roleList']['roles'], basestring):
                    role_list = [xmltodict.parse(response.text)['roleList']['roles']]
                else:
                    role_list = xmltodict.parse(response.text)['roleList']['roles']
            except Exception, e:
                logger.exception(e)
        return role_list

    def assign_to_user(self, username, roles):
        """
        Asign roles to a user
        :param username: user to assign roles
        :param roles: list of roles to assign to the user
        :return: success/failure
        :rtype: list
        """
        if isinstance(roles, basestring):
            raise ValueError("[ERROR] roles parameter must be iterable but not a string...")
        response = self._pentaho.make_call(PENTAHO_ROLES_ENDPOINT_API, ASSIGN_ROLE_TO_USER,
                                           params={USERS_USERNAME_VAR: username,
                                                   ROLES_ASSIGN_VAR: TAB_SEPARATOR.join(roles)})
        return response.status_code == 200

    def remove_from_user(self, username, roles):
        """
        Remove roles from a user
        :param username: user to assign roles
        :param roles: list of roles to remove from the user
        :return: success/failure
        :rtype: list
        """
        if isinstance(roles, basestring):
            raise ValueError("[ERROR] roles parameter must be iterable but not a string...")
        response = self._pentaho.make_call(PENTAHO_ROLES_ENDPOINT_API, REMOVE_ROLE_FROM_USER,
                                           params={USERS_USERNAME_VAR: username,
                                                   ROLES_ASSIGN_VAR: TAB_SEPARATOR.join(roles)})
        return response.status_code == 200

    def create(self, rolename):
        """
        Create a new role
        :param rolename: role to create
        :return: success/failure
        :rtype: boolean
        """
        if not isinstance(rolename, basestring):
            raise ValueError("[ERROR] role name parameter must be a string...")
        response = self._pentaho.make_call(PENTAHO_ROLES_ENDPOINT_API, CREATE_ROLE,
                                           params={ROLE_CREATE_VAR: rolename})
        return response.status_code == 200

    def delete(self, rolenames):
        """
        Delete roles in pentaho
        :param rolenames: the roles to delete (string or list)
        :return: success/failure
        :rtype: boolean
        """
        if type(rolenames) is list:
            rolenames = TAB_SEPARATOR.join(rolenames)
        response = self._pentaho.make_call(PENTAHO_ROLES_ENDPOINT_API, DELETE_ROLE,
                                           params={ROLE_DELETE_VAR: rolenames})
        return response.status_code == 200

    def list(self, regex=None):
        """
        Get list of all roles
        :param regex: filter pattern
        :return: list of roles
        :rtype: list
        """
        role_list = list()
        response = self._pentaho.make_call(PENTAHO_ROLES_ENDPOINT_API, LIST_ROLES)
        if response.status_code == 200:
            try:
                role_list = xmltodict.parse(response.text)['roleList']['roles']
            except Exception, e:
                logger.exception(e)
        if regex:
            role_list = [v for v in role_list if re.match(regex, v)]
        return role_list

    def list_members(self, rolename):
        """
        Get list of all users for this role
        :param rolename: role to list users
        :return: list of users
        :rtype: list
        """
        user_list = list()
        response = self._pentaho.make_call(PENTAHO_ROLES_ENDPOINT_API, LIST_MEMBERS_ROLE,
                                           params={ROLE_LIST_MEMBERS_VAR: rolename})
        if response.status_code == 200:
            try:
                user_list = xmltodict.parse(response.text)['userList']['users']
            except Exception, e:
                logger.exception(e)
        return user_list

    def assign_permissions_to_role(self, rolename, manage=False, read=False,
                                   publish=False, create=False, execute=False,
                                   manage_datasource=False,
                                   manage_security=False):
        """
        Assign permission to a role
        :param rolename: role to assign permission
        :param manage:
        :param read:
        :param publish:
        :param create:
        :param execute:
        :param manage_datasource:
        :param manage_security:
        :return: success/failure
        :rtype: boolean
        """
        logical_role_list = []
        if manage:
            logical_role_list.append("org.pentaho.scheduler.manage")
        if read:
            logical_role_list.append("org.pentaho.repository.read")
        if publish:
            logical_role_list.append("org.pentaho.security.publish")
        if create:
            logical_role_list.append("org.pentaho.repository.create")
        if execute:
            logical_role_list.append("org.pentaho.repository.execute")
        if manage_datasource:
            logical_role_list.append("org.pentaho.platform.dataaccess.datasource.security.manage")
        if manage_security:
            logical_role_list.append("org.pentaho.security.administerSecurity")

        json_payload = {
            "assignments": [
                {
                    "roleName": rolename,
                    "logicalRoles": logical_role_list
                }
            ]
        }
        if not isinstance(rolename, basestring):
            raise ValueError("[ERROR] role name parameter must be a string...")
        response = self._pentaho.make_call(PENTAHO_ROLES_ENDPOINT_API, ASSIGN_PERM_TO_ROLE,
                                           json=json_payload)
        return response.status_code == 200

    def get_permissions_role_map(self, locale='en'):
        """
        Get the permission role map in a dictionary format
        :param locale: locale for the permssion, 'en' by default
        :return: Dictionary of the permissions by role, or None
        """
        if not isinstance(locale, basestring):
            raise ValueError("[ERROR] locale parameter must be a string...")
        response = self._pentaho.make_call(PENTAHO_ROLES_ENDPOINT_API, LIST_PERM_ROLE_MAP,
                                           params={ROLE_MAP_LOCALE_VAR: locale})
        if response.status_code == 200:
            try:
                perm_role_map = xmltodict.parse(response.text)
            except Exception, e:
                return None
            return perm_role_map.get('systemRolesMap', None)
        return None



import os

import pytest
import requests_mock

from penapi.pentaho import Pentaho
from tests import FIXTURES_DIR


@pytest.fixture
def list_directory():
    with open(os.path.join(FIXTURES_DIR, 'list_directory.xml'), 'r') as fixture_file:
        data = fixture_file.read().replace('\n', '')
    return data


@pytest.fixture
def list_acl():
    with open(os.path.join(FIXTURES_DIR, 'list_acl.xml'), 'r') as fixture_file:
        data = fixture_file.read().replace('\n', '')
    return data


@pytest.fixture
def pentaho():
    pentaho = Pentaho(pentaho_base_url='http://test.com')
    return pentaho


@pytest.fixture
def pentaho_acl_dict():
    return {
        "entriesInheriting": False,
        "owner": "test@test.com",
        "ownerType": "1",
        "aces": [
            {
                "modifiable": True,
                "recipientType": 1,
                "permissions": [ 0 ],
                "recipient": "Authenticated"
            },
            {
                "modifiable": True,
                "recipientType": 1,
                "permissions": [ 4 ],
                "recipient": "Administrator"
            }
        ]
    }


@requests_mock.Mocker(kw='mock_api')
def test_files_api_list_success(mock_api, pentaho=pentaho()):
    mock_api.get('http://test.com/pentaho/api/repo/files/:home:dev:demo/tree?depth=3', text=list_directory())
    dir_tree = pentaho.files.list_directory("/home/dev/demo", depth=3)
    assert len(dir_tree['children']) == 8


@requests_mock.Mocker(kw='mock_api')
def test_files_api_list_acl_success(mock_api, pentaho=pentaho()):
    mock_api.get('http://test.com/pentaho/api/repo/files/:home:dev:demo/acl', text=list_acl())
    acl = pentaho.files.get_acl("/home/dev/demo")
    assert len(acl)


@requests_mock.Mocker(kw='mock_api')
def test_files_api_create_directory(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/repo/dirs/:home:dev:demo', text='', status_code=200)
    success = pentaho.files.create_directory("/home/dev/demo")
    assert success


@requests_mock.Mocker(kw='mock_api')
def test_files_api_create_directory_exist(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/repo/dirs/:home:dev:demo', text='', status_code=409)
    success = pentaho.files.create_directory("/home/dev/demo")
    assert success


@requests_mock.Mocker(kw='mock_api')
def test_files_api_create_directory_fail(mock_api, pentaho=pentaho()):
    mock_api.put('http://test.com/pentaho/api/repo/dirs/:home:dev:demo', text='', status_code=500)
    success = pentaho.files.create_directory("/home/dev/demo")
    assert not success


@requests_mock.Mocker(kw='mock_api')
def test_files_api_set_acl_success(mock_api, pentaho=pentaho(), pentaho_acl_dict=pentaho_acl_dict()):
    mock_api.put('http://test.com/pentaho/api/repo/files/:home:dev:demo/acl', text='', status_code=200)
    success = pentaho.files.set_acl("/home/dev/demo", pentaho_acl_dict)
    assert success

